'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.buildConnectOptions = buildConnectOptions;
exports["default"] = connect;

var _connectAdvanced = require('../components/connectAdvanced');

var _connectAdvanced2 = _interopRequireDefault(_connectAdvanced);

var _mapDispatchToProps = require('./mapDispatchToProps');

var _mapDispatchToProps2 = _interopRequireDefault(_mapDispatchToProps);

var _mapStateToProps = require('./mapStateToProps');

var _mapStateToProps2 = _interopRequireDefault(_mapStateToProps);

var _mergeProps = require('./mergeProps');

var _mergeProps2 = _interopRequireDefault(_mergeProps);

var _selectorFactory = require('./selectorFactory');

var _selectorFactory2 = _interopRequireDefault(_selectorFactory);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/*
  connect is a facade over connectAdvanced. It turns its args into a compatible
  selectorFactory, which has the signature:

    (dispatch, options) => (nextState, nextOwnProps) => nextFinalProps
  
  connect passes its args to connectAdvanced as options, which will in turn pass them to
  selectorFactory each time a Connect component instance is instantiated or hot reloaded.

  selectorFactory returns a final props selector from its mapStateToProps,
  mapStateToPropsFactories, mapDispatchToProps, mapDispatchToPropsFactories, mergeProps,
  mergePropsFactories, and pure args.

  The resulting final props selector is called by the Connect component instance whenever
  it receives new props or store state.
 */

function match(arg, factories) {
  for (var i = factories.length - 1; i >= 0; i--) {
    var result = factories[i](arg);
    if (result) return result;
  }
  return undefined;
}

function buildConnectOptions(mapStateToProps, mapDispatchToProps, mergeProps) {
  var _ref = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  var _ref$mapStateToPropsF = _ref.mapStateToPropsFactories;
  var mapStateToPropsFactories = _ref$mapStateToPropsF === undefined ? _mapStateToProps2["default"] : _ref$mapStateToPropsF;
  var _ref$mapDispatchToPro = _ref.mapDispatchToPropsFactories;
  var mapDispatchToPropsFactories = _ref$mapDispatchToPro === undefined ? _mapDispatchToProps2["default"] : _ref$mapDispatchToPro;
  var _ref$mergePropsFactor = _ref.mergePropsFactories;
  var mergePropsFactories = _ref$mergePropsFactor === undefined ? _mergeProps2["default"] : _ref$mergePropsFactor;
  var _ref$selectorFactory = _ref.selectorFactory;
  var selectorFactory = _ref$selectorFactory === undefined ? _selectorFactory2["default"] : _ref$selectorFactory;
  var _ref$pure = _ref.pure;
  var pure = _ref$pure === undefined ? true : _ref$pure;
  var _ref$__ENABLE_SECRET_ = _ref.__ENABLE_SECRET_EXPERIMENTAL_FEATURES_DO_NOT_USE_OR_YOU_WILL_BE_FIRED;

  var __ENABLE_SECRET_EXPERIMENTAL_FEATURES_DO_NOT_USE_OR_YOU_WILL_BE_FIRED = _ref$__ENABLE_SECRET_ === undefined ? false : _ref$__ENABLE_SECRET_;

  var options = _objectWithoutProperties(_ref, ['mapStateToPropsFactories', 'mapDispatchToPropsFactories', 'mergePropsFactories', 'selectorFactory', 'pure', '__ENABLE_SECRET_EXPERIMENTAL_FEATURES_DO_NOT_USE_OR_YOU_WILL_BE_FIRED']);

  if (!__ENABLE_SECRET_EXPERIMENTAL_FEATURES_DO_NOT_USE_OR_YOU_WILL_BE_FIRED) {
    mapStateToPropsFactories = _mapStateToProps2["default"];
    mapDispatchToPropsFactories = _mapDispatchToProps2["default"];
    mergePropsFactories = _mergeProps2["default"];
    selectorFactory = _selectorFactory2["default"];
    options = { withRef: options.withRef };
  }

  var initMapStateToProps = match(mapStateToProps, mapStateToPropsFactories);
  var initMapDispatchToProps = match(mapDispatchToProps, mapDispatchToPropsFactories);
  var initMergeProps = match(mergeProps, mergePropsFactories);

  return _extends({
    // used in error messages
    methodName: 'connect',

    // used to compute Connect's displayName from the wrapped component's displayName.
    getDisplayName: function getDisplayName(name) {
      return 'Connect(' + name + ')';
    },

    // if mapStateToProps is falsy, the Connect component doesn't subscribe to store state changes
    shouldHandleStateChanges: Boolean(mapStateToProps),

    // passed through to selectorFactory
    selectorFactory: selectorFactory,
    initMapStateToProps: initMapStateToProps,
    initMapDispatchToProps: initMapDispatchToProps,
    initMergeProps: initMergeProps,
    pure: pure

  }, options);
}

function connect() {
  var options = buildConnectOptions.apply(undefined, arguments);
  return (0, _connectAdvanced2["default"])(options.selectorFactory, options);
}