"use strict";

exports.__esModule = true;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// encapsulates the subscription logic for connecting a component to the redux store, as
// well as nesting subscriptions of descendant components, so that we can ensure the
// ancestor components re-render before descendants
var Subscription = function () {
  function Subscription(store, parentSub) {
    _classCallCheck(this, Subscription);

    this.subscribe = parentSub ? parentSub.addNestedSub.bind(parentSub) : store.subscribe;

    this.unsubscribe = null;
    this.nextListeners = this.currentListeners = [];
  }

  Subscription.prototype.ensureCanMutateNextListeners = function ensureCanMutateNextListeners() {
    if (this.nextListeners === this.currentListeners) {
      this.nextListeners = this.currentListeners.slice();
    }
  };

  Subscription.prototype.addNestedSub = function addNestedSub(listener) {
    this.trySubscribe();

    var isSubscribed = true;
    this.ensureCanMutateNextListeners();
    this.nextListeners.push(listener);

    return function unsubscribe() {
      if (!isSubscribed) return;
      isSubscribed = false;

      this.ensureCanMutateNextListeners();
      var index = this.nextListeners.indexOf(listener);
      this.nextListeners.splice(index, 1);
    };
  };

  Subscription.prototype.notifyNestedSubs = function notifyNestedSubs() {
    var listeners = this.currentListeners = this.nextListeners;
    var length = listeners.length;
    for (var i = 0; i < length; i++) {
      listeners[i]();
    }
  };

  Subscription.prototype.isSubscribed = function isSubscribed() {
    return Boolean(this.unsubscribe);
  };

  Subscription.prototype.trySubscribe = function trySubscribe() {
    if (!this.unsubscribe) {
      this.unsubscribe = this.subscribe(this.onStateChange);
    }
  };

  Subscription.prototype.tryUnsubscribe = function tryUnsubscribe() {
    if (this.unsubscribe) {
      this.unsubscribe();
    }
    this.unsubscribe = null;
  };

  return Subscription;
}();

exports["default"] = Subscription;